DAT programName         byte "EasyVrDemoLarge131025f", 0
CON
{{
  ******* Public Notes *******
  Program to demonstrate the use of the
  EasyVR voice recognition module.
  The program doesn't not require input
  from a terminal to work.
  The program starts listening for
  word group #2. The word "LED" will
  allow one to select the color of a RGB
  LED. The word "other" will activate
  wordset #1. The LED menu may be reached
  by saying the word "hello" when the
  program is listening to wordset #1.
  The words the EasyVR is actively listening
  for are listed with each word spoken.

  Error messages based on the error code
  returned from the module are now
  displayed.

  To add your own words to the program
  you'll need to make the following changes.
  Add the new word to the appropriate group
  in the DAT section. This word will also
  need to be added to the EasyVR module
  with the GUI program on a PC.
  Follow the example of the other words
  when adding required zeros between
  words.
  , 0[WORDSET_LABEL_SIZE - x]
  The number used for "x" should be
  the number of letters in the new
  word.
  You may want to add a constant to identify
  an action to be taken. The action constants
  are between NO_NEXT_ACTION and LED_NEXT.
  It may be easier to use an previously
  existing action constant and just
  modify the "TakeAction" method with the
  desired action.
  See the examples of added LED control
  and servo control.
  

  Possible Improvements:
  Add sensitivity settings.
  Add ablility to retrain words.
  Add ablility to enter words to
  DAT section of program and have the
    Propeller upload word to module and
    prompt for training.
  Add other baud settings for
    EasyVR.
  Add EEPROM backup of setting
    changes.
    
}}
{
  ******* Private Notes *******

  Modified from MecanumRemote110603a.spin

  110604a Start making a driver for the
  Parallax Say It Module.  110605a test
  skeleton program.  5a doesn't work.
  Say only returns $FF.  b switch tx and
  rx.  b works to wake up Say It but that's
  all it does.  c add ability to same
  parameters to EEPROM.
  f the terminal showed
  "Say It returned =s
  Say It returned =A "
  After I said "robot".  But only once.
  Partial success.
  06d Allow bridge to share debug line.
  Switch serial driver to non-flag version.
  e remove any zeros from received characters.
  I added a pull-up resistor on P27.  It might
  have made things worse.
  09b The voice part works okay.
  Remember to wake the robot up.
  The bridge through the debug line doesn't
  seem to work.
  130302a Update program to use only two serial
  lines.
  02b GUI and terminal window can not share a
  com port so a third line would be needed
  to monitor data exchange between PC and
  EasyVr.
  02d Remove bridge portion of program. Switching from terminal to
  GUI causes Prop to reset so bridge doesn't work.
  Use SayItSimpleBridge.spin as bridge when needed.
  05a Add speaker dependent commands. The commands need to be
  programmed into the SayIt using the PC GUI.

  Change name from SayItAlpha130323a to EasyVrSmall131018a.
  Simplify program.
  131019b Lots of changes made. Tried running it but timeout not set.
  Strange characters received when attemping firmware ID.
  19d Attempt adding speaker dependent words.
  19d Does not work with SD words.  
  19e Add better error handling.
  Change name from EasyVrSmall131020d to EasyVrDemoLarge131020d since
  I haven't yet reduced the programs size.
  131020d SD words work.
  20d Try to clean up the debug a bit.
  20e Kind of works.
  20f Try to make actions more consistent.
  20f Moved LED call to action method.
  EasyVrDemoLarge131023a
  23a Add servo.
  23a Change the way numeric input is handled.
  23a Change numeric input variable names.
  23a Add lookup table "numericActionType". I don't really like
  this feature. I think it may make it more difficult to
  modify the program rather than making it easier.
  23b Abandon "numericActionType" table.
  23c Reduce number of "numericActionType" possibilities.
  Use "inputDigitsCountdown" to indicate how many digits
  to use in fixed digit inputs.
  23d Add "afterActionListeningMode" variable.
  23e Hide use of "afterActionListeningMode" so someone
  changing the program doesn't need to worry about it.
  23e List of possible words appears twice. Move right
  action didn't execute correctly. It still required
  a number.
  23f Apparently the code requests a number after
  move right command. Oops. Fix command.
  24a Found bug in "Init" method. I haven't tested
  the fix yet.
  24a "WakeUp" works but plan to fix anyway. Display
  data exchange.
  24b Program is working reasonably well. Lots of problems
  with menu items. Firmware version will only work if the
  module is awakened again. The wake section watches for
  the wrong character. Lots of little problems but they
  look easy to fix. The program is coming along.
  24b The list of active words is displayed twice. I
  haven't figured out why yet.
  24d Program seems to be working. Keep making minor
  changes.
  24e Add error handling. Not tested.
  25b Only display active word list when the wordset
  or group has changed.
  25b Error after speaking "LED". It looks like the Prop
  is outputing characters from the wrong section of memory.
  25c Setting active word twice for some reason.
  25d I think the problem with setting the active word too
  many times is due to the location of the "SetListeningMode"
  calls. The call isn't needed in the "FindBranch" method
  since the "FindBranch" method returns the value of the
  next mode which is again set by the "ProcessReply" method.
  25d Program isn't retrieving the text of group words
  from EasyVR module correctly.
  25d Posted to forum with one download before I pulled
  the file.
  25f Clean up code a little bit more and rerelease to
  forum.
  
   
}
CON

 
  ' Protocol Command
  CMD_BREAK = "b" ' abort recog or ping
  CMD_SLEEP = "s" ' go to power down
  CMD_KNOB = "k" ' set si knob <1>
  CMD_LEVEL = "v" ' set sd level <1>
  CMD_LANGUAGE = "l" ' set si language <1>
  CMD_TIMEOUT = "o" ' set timeout <1>
  CMD_RECOG_SI = "i" ' do si recog from ws <1>
  CMD_RECOG_SD = "d" ' do sd recog at group <1> (0 = trigger mixed si/sd)
  CMD_ID = "x"
  CMD_DUMP_SD = "p"
  CMD_MASK_SD = "m"
  CMD_COUNT_SD = "c"
  
  ' Protocol Status
  STS_AWAKEN = "w" ' back from power down mode
  STS_ERROR = "e" ' signal error code <1-2>
  STS_INVALID = "v" ' invalid command or argument
  STS_TIMEOUT = "t" ' timeout expired
  STS_INTERR = "i" ' back from aborted recognition (see 'break')
  STS_SUCCESS = "o" ' no errors status
  STS_RESULT = "r" ' recognised sd command <1> - training similar to sd <1>
  STS_SIMILAR = "s" ' recognised si <1> (in mixed si/sd) - training similar to si <1>
 
  ' Protocol arguments are in the range 0x40 (-1) TO 0x60 (+31) inclusive
  ARG_MIN = 64 ' 0x40
  ARG_MAX = 96 ' 0x60
  ARG_ZERO = 65 ' 0x41
   
  ARG_ACK = 32 ' 0x20    'TO READ more status arguments


  ' language enumeration
  #0, ENGLISH', _Italian, _Japanese, _German  
   
CON

  _Clkmode = xtal1 + pll16x     ' 80MHz  Change these two settings to match
  _Xinfreq = 5_000_000          ' your setup.


  ' Pin Assignments
  RED_LED = 0
  GREEN_LED = 1
  BLUE_LED = 2                 
                                
  EASYVR_TX = 10                 ' Connected to RTX on EasyVR
  EASYVR_RX = 11                 ' Connected to ETX on EasyVr 
  DEBUG_TX = 30
  DEBUG_RX = 31

  SERVO_PIN = 15 
  RIGHT_SERVO_PULSE = 1000 
  CENTER_SERVO_PULSE = 1500 
  
  EASYVR_BAUD = 9600
  DEBUG_BAUD = 115_200  

  WORDSET_LABEL_SIZE = 11 ' limit words to ten characters (same limit as GUI)
  LISTENING_TEXT_SIZE = 21
  BASE_TEXT_SIZE = 11
  DEFAULT_WAKE_ATTEMPTS = 8
  TIMEOUT_ATTEMPTS = 3
  LANGUAGE_ATTEMPTS = TIMEOUT_ATTEMPTS
  
  SECONDS_TO_WAIT = 4
  RX_TIME_WAIT = 500
  ATTEMPT_DELAY = 40_000_000
  
  MAX_ERROR_CHARACTERS = 2
  ERROR_BUFFER_SIZE = MAX_ERROR_CHARACTERS + 1
  TEMP_BUFFER_SIZE = 40
  'WORDSET_ATTEMPTS = 8
  'TX_DELAY = 80_000
  QUOTE = 34
  'I_SET_SIZE_0 = 1
  'I_SET_SIZE_1 = 8
  'I_SET_SIZE_2 = 6
  'I_SET_SIZE_3 = 11

  'GROUP_SIZE_1 = 2
  'GROUP_SIZE_2 = 7
  'GROUP_SIZE_3 = 0
  'GROUP_SIZE_PASSWORD = 1
  NOT_LISTENING_SIZE = 1

  LED_ON = 0    ' cathode controlled (common anode) LED use "1" for anode controlled LEDs
  LED_OFF = 1   ' switch these values for common cathode LEDs
    
  ' communication enumeration
  #0, EASYVR_COM, DEBUG_COM

  ' listeningMode enumeration
  #0, LISTEN_FOR_WORDSET_0, LISTEN_FOR_WORDSET_1, LISTEN_FOR_WORDSET_2, {
    } LISTEN_FOR_WORDSET_3, LISTEN_FOR_GROUP_1, LISTEN_FOR_GROUP_2, LISTEN_FOR_GROUP_3, {
    } LISTEN_FOR_GROUP_4, LISTEN_FOR_GROUP_5, LISTEN_FOR_GROUP_6, LISTEN_FOR_GROUP_7, {
    } LISTEN_FOR_GROUP_8, LISTEN_FOR_GROUP_9, LISTEN_FOR_GROUP_10, LISTEN_FOR_GROUP_11, {
    } LISTEN_FOR_GROUP_12, LISTEN_FOR_GROUP_13, LISTEN_FOR_GROUP_14, LISTEN_FOR_GROUP_15, {
    } LISTEN_FOR_PASSWORD, LISTEN_FOR_NOTHING

  INITIAL_MODE = LISTEN_FOR_GROUP_1
  LAST_LISTENING_MODE_INDEX = LISTEN_FOR_NOTHING
  LISTENING_MODES = LAST_LISTENING_MODE_INDEX + 1

  ' wordset1 enumeration
  #0, ACTION_WS1, MOVE_WS1, TURN_WS1, RUN_WS1, LOOK_WS1, ATTACK_WS1, STOP_WS1, HELLO_WS1

  ' wordset2 enumeration
  #0, LEFT_WS2, RIGHT_WS2, UP_WS2, DOWN_WS2, FORWARD_WS2, BACKWARD_WS2

  ' wordset3 enumeration
  #0, ZERO_WS3, ONE_WS3, TWO_WS3, THREE_WS3, FOUR_WS3, FIVE_WS3, SIX_WS3, SEVEN_WS3, {
    } EIGHT_WS3, NINE_WS3, TEN_WS3

  ' group1 enumeration
  #0, LED_G1, OTHER_G1

  ' group2 enumeration
  #0, RED_G2, YELLOW_G2, GREEN_G2, BLUE_G2, VIOLET_G2, OFF_G2, EXIT_G2

  '' Add other group enumerations here.
  
  ' next action enumeration 
  #0, NO_NEXT_ACTION, ACTION_NEXT, MOVE_LEFT_NEXT, MOVE_RIGHT_NEXT, {
    } MOVE_UP_NEXT, MOVE_DOWN_NEXT, MOVE_FORWARD_NEXT, MOVE_BACKWARD_NEXT, {
    } TURN_LEFT_NEXT, TURN_RIGHT_NEXT, TURN_UP_NEXT, TURN_DOWN_NEXT, {
    } TURN_FORWARD_NEXT, TURN_BACKWARD_NEXT, RUN_NEXT, LOOK_LEFT_NEXT, {
    } LOOK_RIGHT_NEXT, LOOK_UP_NEXT, LOOK_DOWN_NEXT, LOOK_FORWARD_NEXT, {
    } LOOK_BACKWARD_NEXT, ATTACK_LEFT_NEXT, ATTACK_RIGHT_NEXT, ATTACK_UP_NEXT, {
    } ATTACK_DOWN_NEXT, ATTACK_FORWARD_NEXT, ATTACK_BACKWARD_NEXT, STOP_NEXT, {
    } HELLO_NEXT, LED_NEXT

  ' numericInputType enumeration
  #0, NO_NUMERIC_INPUT, FIXED_DIGIT_INPUT, TEN_TERMINATED_INPUT
  
VAR
 
  long cog
  long numericInputTotal 'wordset3Total ' renamed 10/23
  long activeWordsPtr
 
  byte listeningMode
  byte previousIndex, priorPreviousIndex
  byte inputDigitsCountdown 'wordset3DigitRemaining ' renamed 10/23
                            ' this variable may not be needed at all.
                            ' "numericInputType" could serve as countdown
                            
  byte nextAction 
  byte similarFlag, activeWords, errorBuffer[ERROR_BUFFER_SIZE]
  byte tempBufferTx[TEMP_BUFFER_SIZE], tempBufferRx[TEMP_BUFFER_SIZE]
  byte numericInputType ' added 10/23
  byte afterActionListeningMode ' added 10/23
  
OBJ

  EasyVr : "Parallax Serial Terminal"
  Debug : "Parallax Serial Terminal"
  Servo : "Servo32v9" ' added 10/23
   
PUB Setup | localCharacter, localIndex, {
             } localAttmpts

  nextAction := NO_NEXT_ACTION
               
  outa[RED_LED] := LED_OFF
  outa[GREEN_LED] := LED_OFF
  outa[BLUE_LED] := LED_OFF
  dira[RED_LED] := 1
  dira[GREEN_LED] := 1
  dira[BLUE_LED] := 1
 
  
  EasyVr.StartRxTx(EASYVR_RX, EASYVR_TX, 0, EASYVR_BAUD)
  Debug.Start(DEBUG_BAUD)

  Debug.Str(string(13, "Com driver started"))
  Servo.Start 
  FillSetSizeTable
  repeat
    Debug.Str(string(13, "Press any key to start"))
    localIndex := Debug.RxCount
    waitcnt(clkfreq / 2 + cnt)
  while localIndex == 0 and result++ < SECONDS_TO_WAIT * 2
  Debug.Char(13)
  Debug.Char(13)
  Debug.RxFlush
  
  localIndex := 0
  localAttmpts := 0
  ' I'm not consistent with where communication attempts are
  ' made. The wake up attempts are attempted from this
  ' method while other communication attempts are made from
  ' one of the specialized communication methods.
  ' I've been cleaning up the code but it still has
  ' some quirks.
  localIndex := WakeUp(DEFAULT_WAKE_ATTEMPTS) 
    
  localIndex := SetTimeout(0)
  if localIndex == 0      
    Debug.Str(string(13, "Set timeout failed."))
    Debug.Str(string(13, "Continuing anyway."))

  else
    Debug.Str(string(13, "Timeout setting succeeded."))
    ' I'm not sure if this is true.
    
  Debug.Str(string(13, "Initializing EasyVR Module"))
  localIndex := Init  ' This sets the language. It isn't really needed.
  if localIndex == 0      
    Debug.Str(string(13, "Initializing failed."))
    Debug.Str(string(13, "Continuing anyway."))
    Debug.Str(string(13, "This step isn't really needed anyway."))
  else
    Debug.Str(string(13, "Initializing succeeded."))

  SetListeningMode(INITIAL_MODE) ' INITIAL_MODE is set in constants section
              
  MainLoop
  
PUB MainLoop | localCharacter, localIndex, localAttmpts
'' This loop looks for input from two places.
'' Input from a terminal program and input from
'' the EasyVR.
'' The input from the EasyVR should mostly be from
'' automatic replies based on words spoken.
'' Communication from the EasyVR based on requests
'' for information takes place in sub methodes.
'' Characters received from a terminal program
'' may trigger various menu actions.

  similarFlag := 0         ' this is set to one when the next
                                ' character read in should be a
                                ' wordset index number.
                              
  DisplayTopMenu
  
  repeat
    result := EasyVr.RxCount
    
    if result <> 0
      result := ReceiveReply
      Debug.Str(string(13, "EasyVR returned = "))
      'Debug.Char(localCharacter)
      Str(@tempBufferRx)
      ProcessReply(result)
               
    localCharacter := Debug.RxCount
    
    if localCharacter <> 0
      localCharacter := Debug.CharIn
      case localCharacter
        "x", "X", "m", "M", "c", "C", "p", "P", "z", "Z":
          Request(localCharacter)
        "w", "W":
          WakeUp(DEFAULT_WAKE_ATTEMPTS)        
        "i", "I":
          Debug.Str(string(13, "Initializing EasyVR Module"))
          localIndex := Init
          Debug.Str(string(13, "Init method returned "))
          Debug.Dec(localIndex)    
          Debug.Str(string(" errors."))
        "d", "D":
          DisplayTopMenu
        "o", "O":
          SetTimeout(-1) ' require input
          Debug.Str(string(13, "Initializing EasyVR Module"))
          Init
        "t", "T":
          Debug.Str(string(13, "EasyVR is now listening for trigger."))
          ClearActionVariables
          SetListeningMode(LISTEN_FOR_WORDSET_0)
        "s", "S":
          Debug.Str(string(13, "Test speaker dependent commands."))
          Debug.Str(string(13, "Starting from group 1."))
          ClearActionVariables
          SetListeningMode(LISTEN_FOR_GROUP_1) 
        other:
          Debug.Str(string(13, QUOTE)) 
          Debug.Char(localCharacter) 
          Debug.Str(string(QUOTE, " was entered.  Which is not a menu choice.")) 
      DisplayTopMenu

PUB ClearActionVariables

  nextAction := NO_NEXT_ACTION
  numericInputType := NO_NUMERIC_INPUT 
  numericInputTotal := 0 
  inputDigitsCountdown := 0
                            
PRI ProcessReply(localSize) : newMode | localIndex

  newMode := listeningMode
  Debug.Str(string(13, "ProcessReply("))
  Debug.Dec(localIndex)
  Debug.Char(")")
  Debug.Str(string(13, "listeningMode = "))
  Debug.Dec(listeningMode)
  Debug.Str(string(" = ", QUOTE))
  Debug.Str(GetListeningModeStr)
  localIndex := 0
  repeat localSize
    Debug.Str(string(13, "tempBufferRx["))
    Debug.Dec(localIndex)
    Debug.Str(string("] = "))
    SafeTx(tempBufferRx[localIndex])
    if similarFlag == 1
      Debug.Str(string(13, "similarFlag == 1"))
      'Debug.Str(string(13, "tempBufferRx["))
      'Debug.Dec(localIndex)
      'Debug.Str(string("] = "))
      'Debug.Dec(tempBufferRx[localIndex])
      newMode := FindBranch(tempBufferRx[localIndex])
      similarFlag := 0
      if localIndex > 2
        Debug.Str(string(13, "An unexpected character was returned."))
      quit
    elseif tempBufferRx[localIndex] == STS_SIMILAR or tempBufferRx[localIndex] == STS_RESULT
      localIndex++
      similarFlag := 1
      Debug.Str(string(13, "similarFlag set"))
      Debug.Str(string(13, "localIndex = "))
      Debug.Dec(localIndex)
      
    elseif tempBufferRx[localIndex] == STS_TIMEOUT
      Debug.Str(string(13, "Timeout error was returned."))
      quit 
    elseif tempBufferRx[localIndex] == STS_ERROR
      Debug.Str(string(13, "An error was returned."))
      HandleError
      quit 
  Debug.Str(string(13, "Returning to previous listening mode."))  
  SetListeningMode(newMode)  
      
PRI HandleError

  if tempBufferRx[0] == "e"
    result := ((tempBufferRx[1] - ARG_ZERO) << 4) + (tempBufferRx[2] - ARG_ZERO)
    Debug.Str(string(13, 13, "   **** Error $"))
    Debug.Hex(result, 2)
    Debug.Str(string(" : "))
    case result
      $03:
        Debug.Str(string("too noisy"))  
      $04:
        Debug.Str(string("spoke too soft"))  
      $05:
        Debug.Str(string("spoke too loud"))  
      $06:
        Debug.Str(string("spoke too soon"))  
      $07:
        Debug.Str(string("too complex"))  
      $11:
        Debug.Str(string("word not recognized"))  
      $12:
        Debug.Str(string("low confidence recognition"))  
      $13:
        Debug.Str(string("medium confidence recognition"))  
      $14:
        Debug.Str(string("invalid command stored"))  
      $17:
        Debug.Str(string("bad pattern durations"))  
      $4A:
        Debug.Str(string("bad version number?"))  
      $4E:
        Debug.Str(string("bad data in speach file"))  
      $80:
        Debug.Str(string("not a word"))  
    Debug.Str(string(" ****", 13))
    
PUB DisplayTopMenu | localIndex, localWordSet, localPtr

  Debug.Str(string(13, "programName = ", QUOTE))
  Debug.Str(@programName)
  Debug.Char(QUOTE)
  
  Debug.Str(string(13, "listeningMode = ", QUOTE))
  'Debug.Str(@listeningText + (listeningMode * LISTENING_TEXT_SIZE))
  Debug.Str(GetListeningModeStr)
  Debug.Char(QUOTE)
  'Debug.Str(string(13, "Actively listening for following words:"))
  'DisplayActiveWords
 
  Debug.Char(13)
  Debug.Str(string(13, "Enter ", QUOTE, "d", QUOTE, " to display this menu.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "w", QUOTE, " to wake up EasyVR module.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "i", QUOTE, " to initialize EasyVR module.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "x", QUOTE, " to request firmware ID of module.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "o", QUOTE, " to set timeout of module.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "m", QUOTE, " to request non-empty groups of module.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "c", QUOTE, " to request count of speaker dependent commands.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "p", QUOTE, " to request speaker dependent commands of module.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "t", QUOTE, " to have EasyVR listen for trigger.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "s", QUOTE, " to test speaker dependent commands.")) 
  Debug.Str(string(13, "Enter ", QUOTE, "z", QUOTE, " to compare text of speaker dependent commands.")) 
  Debug.Str(string(13, "          (Compares text on EasyVR with text on Propeller.")) 
  Debug.Char(13)
  Debug.Char(13)
  Debug.Char(13)
  
PUB Request(localCharacter) | localTemp[2]

  TxEvrOneCharacter(CMD_BREAK, 1) ' Stop listening for word 
  ' This might not work on old SayIt or EasyVR modules but it should
  ' for any purchased in 2013 or later.
        
  case localCharacter
    "x", "X":
      Debug.Str(string(13, "Request firmware ID of module."))
      result := TxEvrOneCharacter(CMD_ID, 2)
      if result
        Debug.Str(string(13, 13, "          **** firmware ID = "))
        Debug.Dec(tempBufferRx[result - 1] - ARG_ZERO)
        Debug.Str(string(" **** ", 13)) 
      else
        Debug.Str(string(13, "No character received."))
        
    "m", "M":
      Debug.Str(string(13, "Request non-empty groups of module."))
      result := TxEvrOneCharacter(CMD_MASK_SD, 2)
      if result
        if tempBufferRx[0] == "k"
          Debug.Str(string(13, "Received appropriate reply from module."))
          Debug.Str(string(13, "Reply = ", QUOTE))
          Str(@tempBufferRx + 1)
          Debug.Char(QUOTE)
          Debug.Str(string(13, "Reply as numbers = "))
          repeat localTemp[0] from 1 to result - 1
            Debug.Dec(tempBufferRx[localTemp[0]] - ARG_ZERO)
            if localTemp[0] <> result - 1
              Debug.Char(",")
              Debug.Char(" ")
          Debug.Str(string(13, "Reply binary mask = %"))
          repeat localTemp[0] from 1 to result - 1
            Debug.Bin(tempBufferRx[localTemp[0]] - ARG_ZERO, 4)
            if localTemp[0] <> result - 1
              Debug.Char(",")
              Debug.Char(" ")
              Debug.Char("%")
        else
          Debug.Str(string(13, "Received inappropriate reply from module."))
          Str(@tempBufferRx)
      else
        Debug.Str(string(13, "No mask received."))
      
    "c", "C":
      Debug.Str(string(13, "Request count of speaker dependent commands."))
      result := GetGroupIndex
      result := TxEvrTwoCharacters(CMD_COUNT_SD, result, 2)
      if result
        Debug.Str(string(13, "SD command count = "))
        Debug.Dec(tempBufferRx[result - 1] - ARG_ZERO) 
      else
        Debug.Str(string(13, "No count received."))

    "p", "P":
      Debug.Str(string(13, "Reading speaker dependent commands."))
      'tempBufferTx[0] := CMD_DUMP_SD
      repeat
        localTemp[0] := GetGroupIndex 
        if localTemp[0] < 1 or localTemp[0] > 16
          Debug.Str(string(13, "Number out of range."))
          localTemp[0] := -1
      while localTemp[0] == -1      
      repeat
        Debug.Str(string(13, "Please enter command position (0-31)."))
        localTemp[1] :=  GetNumber
        if localTemp[1] < 0 or localTemp[1] > 31
          Debug.Str(string(13, "Number out of range."))
          localTemp[1] := -1
      while localTemp[1] == -1
      Debug.Str(string(13, "command position = "))
      Debug.Dec(localTemp[1])         
      result := TxEvrThreeCharacters(CMD_DUMP_SD, localTemp[0], localTemp[1], 2)
      if result
        Debug.Str(string(13, "speaker dependent command in group #"))
        Debug.Dec(localTemp[0])
        Debug.Str(string(", index #")) 
        Debug.Dec(localTemp[1])        
        if strsize(@tempBufferRx) => 4
          Debug.Str(string(" = ", QUOTE)) 
          Str(@tempBufferRx + 4)
        else
          DumpError(localTemp[0], localTemp[1])
      else
        DumpError(localTemp[0], localTemp[1])
    "z":
      repeat localTemp[0] from LISTEN_FOR_GROUP_1 to LISTEN_FOR_PASSWORD
        if iSetSize[localTemp[0]]
          repeat localTemp[1] from 0 to iSetSize[localTemp[0]] - 1  
            CompareStoredWord(localTemp[0] - LISTEN_FOR_WORDSET_3, localTemp[1])         

  WakeUp(DEFAULT_WAKE_ATTEMPTS)
  
PUB DumpError(group, command)

  if tempBufferRx[0]
    Debug.Str(string(13, "Received the following dump: ", QUOTE))
    Str(@tempBufferRx)
    Debug.Char(QUOTE)
  else
    Debug.Str(string(13, "No characters were dumped."))
  if iSetSize[group + LISTEN_FOR_WORDSET_3] == 0
    Debug.Str(string(13, "It's not surprising there wasn't a word."))
    Debug.Str(string(13, "That group should be empty."))
  elseif (command + 1) > iSetSize[group + LISTEN_FOR_WORDSET_3]
    Debug.Str(string(13, "It's not surprising there wasn't a word."))
    Debug.Str(string(13, "The command position was out of range."))
  else
    Debug.Str(string(13, "Expected to receive more characters."))
    Debug.Str(string(13, "Both the group and command position were valid numbers."))
       
PRI CompareStoredWord(localGroup, localPosition) : localPtr | localtemp, adjustedGroup

  'localPtr := @iWordset0 ' set pointer to first word listed in DAT section
                         ' this pinter will be adjusted below
  adjustedGroup := localGroup + LISTEN_FOR_WORDSET_3 'LISTEN_FOR_GROUP_1 '                       
  localPtr := groupAddrTable[adjustedGroup]
  Debug.Str(string(13, 13, "First word in group = ", QUOTE))
  Str(localPtr)
  Debug.Char(QUOTE)
  {repeat localIndex from 0 to adjustedGroup - 1
    localPtr += iSetSize[localIndex] * WORDSET_LABEL_SIZE ' skip words in previous sets
   }
  localPtr += localPosition * WORDSET_LABEL_SIZE                                                                                                                                         
  Debug.Str(string(13, "Group #"))
  Debug.Dec(localGroup)
  Debug.Str(string(", Entry #"))
  Debug.Dec(localPosition)
  localtemp := TxEvrThreeCharacters(CMD_DUMP_SD, localGroup, localPosition, 2)
  Debug.Str(string(" - EasyVR = "))
  if localtemp < 4
    Debug.Str(string("** Unexpected Result ** "))
    Debug.Char(QUOTE)
    Str(@tempBufferRx)
    Debug.Char(QUOTE)
  else  
    Debug.Char(QUOTE)
    Str(@tempBufferRx + 4)
    Debug.Char(QUOTE)
  Debug.Str(string(" Prop = ", QUOTE))
  Str(localPtr)
  Debug.Char(QUOTE)
  if localtemp < 4
    DumpError(localGroup, localPosition)
    
PRI GetGroupIndex

  repeat
    Debug.Str(string(13, "Please enter group index."))
    Debug.Str(string(13, "(1-15 = generic, 16 = password)"))
    Debug.Str(string(13, "Press 'enter' to indicate number has been entered."))
    result :=  GetNumber
  while result == -1
  Debug.Str(string(13, "group index = "))
  Debug.Dec(result)

PUB TxEvrOneCharacter(localCommand, localAttempts)
'' Returns number of characters in reply from EasyVR

  result := TxEvrTwoCharacters(localCommand, -ARG_ZERO, localAttempts)

PUB TxEvrTwoCharacters(localCommand, localParameter, localAttempts) 
'' Returns number of characters in reply from EasyVR

  result := TxEvrThreeCharacters(localCommand, localParameter, -ARG_ZERO, localAttempts)
   
PUB TxEvrThreeCharacters(localCommand, localParameter0, localParameter1, localAttempts) | possibleAttempts
'' Returns number of characters in reply from EasyVR

  tempBufferTx[0] := localCommand
  tempBufferTx[1] := localParameter0 + ARG_ZERO
  tempBufferTx[2] := localParameter1 + ARG_ZERO
  tempBufferTx[3] := 0
  possibleAttempts := localAttempts
  repeat
    EasyVr.Str(@tempBufferTx)
    Debug.Str(string(13, "Attempt "))
    Debug.Dec((possibleAttempts - localAttempts) + 1)
    Debug.Str(string(" of "))
    Debug.Dec(possibleAttempts)
    Debug.Str(string(13, "Sending to EasyVR = ", QUOTE))
    Str(@tempBufferTx)
    Debug.Char(QUOTE) 
    result := ReceiveReply
    if result  
      Debug.Str(string(13, "EasyVR returned = ", QUOTE))
      Str(@tempBufferRx)
      Debug.Char(QUOTE)
      HandleError
    else         
      waitcnt(ATTEMPT_DELAY + cnt)
  while --localAttempts > 0 and result == 0

PUB SetTimeout(localTemp) | localAttempts

  Debug.Str(string(13, "Set Timeout Method"))
  Debug.Str(string(13, "The timeout is the amount of time"))
  Debug.Str(string(13, "the EasyVR module will wait for a"))
  Debug.Str(string(13, "word to be said."))

  if localTemp == -1
    Debug.Str(string(13, "Just press Enter to use default."))
    Debug.Str(string(13, "0 = infinite, 1-30 = seconds"))
     
    Debug.Str(string(13, "Please enter timeout now."))
    Debug.Str(string(13, "(Followed by Enter)"))
    Debug.Str(string(13, "or just Enter for default timeout"))
    Debug.Str(string(13, "(however long the default is?)"))
    localTemp :=  GetNumber
  if localTemp == 0
    Debug.Str(string(13, "Setting infinite timeout."))
  else    
    Debug.Str(string(13, "timeout = "))
    Debug.Dec(localTemp)

  result := TxEvrTwoCharacters(CMD_TIMEOUT, localTemp, TIMEOUT_ATTEMPTS)
  
  if result  
    if tempBufferRx[0] == STS_SUCCESS
      Debug.Str(string(13, "Success!"))
      Debug.Str(string(13, "Timeout set to "))
      Debug.Dec(localTemp)
      Debug.Char(".")   
    else  
      Debug.Str(string(13, "Timeout not set.")) 
      Debug.Str(string(13, "EasyVR returned = ", QUOTE))
      Str(@tempBufferRx)
      Debug.Char(QUOTE)
  else
    Debug.Str(string(13, "Timeout not set."))
    Debug.Str(string(13, "No characters received."))
     
PUB WakeUp(localAttempts)

  result := TxEvrOneCharacter(CMD_BREAK, localAttempts)
 
  if result == 0      
    Debug.Str(string(13, "Wake attempts failed."))
    Debug.Str(string(13, "Continuing anyway."))

  else
    Debug.Str(string(13, "The EasyVR Module is now awake."))
    if tempBufferRx[0] == STS_SUCCESS
      Debug.Str(string(13, "EasyVR was awakened."))
    elseif tempBufferRx[0] == "w"  ' I think that's what this means.
      Debug.Str(string(13, "EasyVR was already awake."))
    else
      Debug.Str(string(13, "The EasyVR is probably awake."))
      Debug.Str(string(13, "The last received text = ", QUOTE))
      Str(@tempBufferRx)
      Debug.Char(QUOTE)
      
PUB Init | localAttempts

 
  Debug.Str(string(13, "Attempting to set English language."))

  bytefill(@tempBufferTx, 0, TEMP_BUFFER_SIZE)
  tempBufferTx[0] := CMD_LANGUAGE
  tempBufferTx[1] := ENGLISH + ARG_ZERO
  localAttempts := 0
  repeat
    EasyVr.Str(@tempBufferTx)
    Debug.Str(string(13, "Set language attempt "))
    Debug.Dec(localAttempts)
    Debug.Str(string(" of "))
    Debug.Dec(LANGUAGE_ATTEMPTS)
    Debug.Str(string(13, "Sending to EasyVR = ", QUOTE))
    Str(@tempBufferTx)
    Debug.Char(QUOTE) 
    result := ReceiveReply
    if result  
      Debug.Str(string(13, "EasyVR returned = ", QUOTE))
      Str(@tempBufferRx)
      Debug.Char(QUOTE)
      
      if tempBufferRx[0] == STS_SUCCESS
        Debug.Str(string(13, "Success!"))
        Debug.Str(string(13, "Language set to English."))
      else  
        Debug.Str(string(13, "Attempt Failed!"))
         
    waitcnt(ATTEMPT_DELAY + cnt)
  while ++localAttempts < LANGUAGE_ATTEMPTS and tempBufferRx[0] <> STS_SUCCESS

PUB FindBranch(localIndex) : nextMode | localPtr, adjustedIndex, localDigit', localErrorCount
'' The listenMode value determines which word set or group is active.
'' The localIndex is the command index indicates which command was just
'' recognized from the active word set or group.
'' Any if "nextAction is set within this method, that action
'' will be executed this loop of the program unless numeric
'' input is required. If numeric input is required, the action
'' will not be taken until numeric input is completed.
'' nextMode does not need to be set if numeric input is
'' required. Setting the variable "numericInputType" to
'' one of the type which require input is enough to make
'' sure groupset #3 will be used.
'' The end of these branches should end in one of three
'' ways.
''
'' One:   Set nextMode to a new wordset or group. The action
''        has not yet been determined.
'' Two:   nextAction is set and nextMode is also set.
''        No numberic input is required.
'' Three: nextAction and nextMode are set.
''        Numeric input is initiated.
''        nextAction is delayed until
''        numeric input is completed.
''        Once numeric input is completed
''        the nextMode is restored to
''        the value set when nextAction
''        was set.
''     

  nextMode := listeningMode
  adjustedIndex := localIndex - ARG_ZERO ' compute the returned number  

  Debug.Str(string(13, "listeningMode = ", QUOTE))
  Debug.Str(GetListeningModeStr)
  Debug.Char(QUOTE)
  localPtr := activeWordsPtr + (adjustedIndex * WORDSET_LABEL_SIZE)
  ' find the word based on the returned number
  
  Debug.Str(string(13, 13, "            EasyVR heard    ****** ", QUOTE)) 
  Str(localPtr)
  Debug.Str(string(QUOTE, " ******    ", 13)) 
  case listeningMode
    LISTEN_FOR_WORDSET_0:  ' listening for trigger
      if adjustedIndex == 0
        nextMode := LISTEN_FOR_WORDSET_1  ' This can be any set or group you want
      else
        DisplayError(localIndex, ARG_ZERO, ARG_ZERO)
    LISTEN_FOR_WORDSET_1:
      case adjustedIndex
        ACTION_WS1:
          nextAction := ACTION_NEXT
          nextMode := LISTEN_FOR_WORDSET_1
          numericInputType := FIXED_DIGIT_INPUT
          inputDigitsCountdown := 1
          numericInputTotal := 0
          Debug.Str(string(13, "Action command received."))
          Debug.Str(string(13, "Waiting for single digit number to execute."))
        MOVE_WS1:
          nextMode := LISTEN_FOR_WORDSET_2 ' This can be any set or group you want
          Debug.Str(string(13, "Move command received."))
          Debug.Str(string(13, "Waiting for direction to move."))
        TURN_WS1:
          nextMode := LISTEN_FOR_WORDSET_2
          Debug.Str(string(13, "Turn command received."))
          Debug.Str(string(13, "Waiting for direction to turn."))
        RUN_WS1:
          nextMode := LISTEN_FOR_WORDSET_1
          nextAction := RUN_NEXT
          numericInputType := FIXED_DIGIT_INPUT
          inputDigitsCountdown := 1
          Debug.Str(string(13, "Run command received."))
          Debug.Str(string(13, "Waiting for program number to run."))
        LOOK_WS1:
          nextMode := LISTEN_FOR_WORDSET_2
          Debug.Str(string(13, "Look command received."))
          Debug.Str(string(13, "Waiting for direction to look."))
        ATTACK_WS1:
          nextMode := LISTEN_FOR_WORDSET_2
          Debug.Str(string(13, "Attack command received."))
          Debug.Str(string(13, "Waiting for direction to attack."))
        STOP_WS1:
          nextMode := LISTEN_FOR_WORDSET_0 ' go back to waiting for trigger
          nextAction := STOP_NEXT
          Debug.Str(string(13, "****** STOP!!! ******", 13))
          Debug.Str(string(13, "(Stop command received.)", 13))
          Debug.Str(string(13, "Waiting for trigger again."))
        HELLO_WS1:
          nextMode := LISTEN_FOR_GROUP_2
          Debug.Str(string(13, 13, "Waiting for LED color."))  
        other:
          DisplayError(localIndex, ARG_ZERO, ARG_ZERO + activeWords - 1)
                            
    LISTEN_FOR_WORDSET_2: 
      case adjustedIndex
        LEFT_WS2:
          'Debug.Str(string(13, "Left command received."))
          Debug.Char(13)
          case previousIndex
            MOVE_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := MOVE_LEFT_NEXT
              Debug.Str(string(13, "Move"))
            TURN_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := TURN_LEFT_NEXT
              Debug.Str(string("Turn"))
            LOOK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := LOOK_LEFT_NEXT   
              Debug.Str(string("Look"))
            ATTACK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := ATTACK_LEFT_NEXT
              Debug.Str(string("Attack")) 
            other:
              DisplayError(localIndex, ARG_ZERO, ARG_ZERO + activeWords - 1)
          Debug.Str(string(" left"))       
        RIGHT_WS2:
          'Debug.Str(string(13, "Right command received."))
          Debug.Char(13)
          case previousIndex
            MOVE_WS1:  
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := MOVE_RIGHT_NEXT
              Debug.Str(string(13, "Move"))
            TURN_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := TURN_RIGHT_NEXT
              Debug.Str(string("Turn"))
            LOOK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := LOOK_RIGHT_NEXT
              Debug.Str(string("Look"))
            ATTACK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := ATTACK_RIGHT_NEXT 
              Debug.Str(string("Attack")) 
            other:
              DisplayError(localIndex, ARG_ZERO, ARG_ZERO + activeWords - 1)
          Debug.Str(string(" right"))               
        UP_WS2:
          'Debug.Str(string(13, "Up command received."))
          Debug.Char(13)
          case previousIndex
            MOVE_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := MOVE_UP_NEXT
              numericInputType := TEN_TERMINATED_INPUT
              Debug.Str(string(13, "Waiting for amount to move up."))
              Debug.Str(string(13, "Move"))
            TURN_WS1:              
              nextAction := TURN_UP_NEXT
              Debug.Str(string("Turn"))
            LOOK_WS1:
              nextAction := LOOK_UP_NEXT 
              Debug.Str(string("Look"))
            ATTACK_WS1:
              nextAction := ATTACK_UP_NEXT
              Debug.Str(string("Attack"))
            other:
              DisplayError(localIndex, ARG_ZERO, ARG_ZERO + activeWords - 1)
          Debug.Str(string(" up"))       
        DOWN_WS2:
          'Debug.Str(string(13, "Down command received."))
          Debug.Char(13) 
          case previousIndex
            MOVE_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := MOVE_DOWN_NEXT
              Debug.Str(string("Move"))
            TURN_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := TURN_DOWN_NEXT
              Debug.Str(string("Turn"))
            LOOK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := LOOK_DOWN_NEXT
              Debug.Str(string("Look"))
            ATTACK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := ATTACK_DOWN_NEXT
              Debug.Str(string("Attack")) 
            other:
              DisplayError(localIndex, ARG_ZERO, ARG_ZERO + activeWords - 1)
          Debug.Str(string(" down"))         
        FORWARD_WS2:
          'Debug.Str(string(13, "Forward command received."))
          Debug.Char(13)
          case previousIndex
            MOVE_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := MOVE_FORWARD_NEXT      
              Debug.Str(string(13, "Move"))
            TURN_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := TURN_FORWARD_NEXT
              Debug.Str(string("Turn"))
            LOOK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := LOOK_FORWARD_NEXT
              Debug.Str(string("Look"))
            ATTACK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := ATTACK_FORWARD_NEXT
              Debug.Str(string("Attack")) 
            other:
              DisplayError(localIndex, ARG_ZERO, ARG_ZERO + activeWords - 1)
          Debug.Str(string(" forward"))    
        BACKWARD_WS2:
          'Debug.Str(string(13, "Backward command received."))
           Debug.Char(13)
          case previousIndex
            MOVE_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := MOVE_BACKWARD_NEXT      
              Debug.Str(string(13, "Move"))
            TURN_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := TURN_BACKWARD_NEXT
              Debug.Str(string("Turn"))
            LOOK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := LOOK_BACKWARD_NEXT
              Debug.Str(string("Look"))
            ATTACK_WS1:
              nextMode := LISTEN_FOR_WORDSET_1
              nextAction := ATTACK_BACKWARD_NEXT 
              Debug.Str(string("Attack")) 
            other:
              DisplayError(localIndex, ARG_ZERO, ARG_ZERO + activeWords - 1)
          Debug.Str(string(" backward"))     
      Debug.Str(string(" command received. Next action has been set."))
    LISTEN_FOR_WORDSET_3: ' lots of changes to this section 10/23  
      if adjustedIndex == TEN_WS3 and numericInputType == TEN_TERMINATED_INPUT
        numericInputType := NO_NUMERIC_INPUT ' done inputing number
      elseif adjustedIndex == TEN_WS3 and numericInputType == FIXED_DIGIT_INPUT
        if inputDigitsCountdown > 1
          numericInputTotal := (100 * numericInputTotal) + 10
          inputDigitsCountdown -= 2
        else
          Debug.Str(string(13, "Ten is not a valid last digit."))
          Debug.Str(string(13, "Please speak a single digit number."))
      else
        numericInputTotal := (10 * numericInputTotal) + adjustedIndex
        Debug.Str(string(13, "digit heard = "))
        Debug.Dec(adjustedIndex)
        
        if numericInputType == FIXED_DIGIT_INPUT
          if --inputDigitsCountdown > 0
            Debug.Str(string(13, "currently numericInputTotal = "))
            Debug.Dec(numericInputTotal)
            Debug.Str(string(13, "Please speak "))
            Debug.Dec(inputDigitsCountdown)
            Debug.Str(string(" more digits."))  
          else
            Debug.Str(string(13, "final numericInputTotal = "))
            Debug.Dec(numericInputTotal)
            numericInputType := NO_NUMERIC_INPUT
        else
          Debug.Str(string(13, "To enter another digit please"))
          Debug.Str(string(13, "speak a single digit number."))
          Debug.Str(string(13, 13, "                 Say ", QUOTE))
          Debug.Str(string("ten", QUOTE, " to end numeric input.", 13))
            
    LISTEN_FOR_GROUP_1:      
      case adjustedIndex
        LED_G1:
          nextMode := LISTEN_FOR_GROUP_2
          Debug.Str(string(13, "Waiting for LED color."))
        OTHER_G1:
          nextMode := LISTEN_FOR_WORDSET_1
          Debug.Str(string(13, "Waiting for robot control word."))
     
    LISTEN_FOR_GROUP_2:      
      case adjustedIndex
        RED_G2..OFF_G2:
          nextMode := LISTEN_FOR_GROUP_2
          nextAction := LED_NEXT
          'LedControl(adjustedIndex)
        'YELLOW_G2..OFF_G2:
          'LedControl(adjustedIndex) ' ** not good flow control
        EXIT_G2:
          nextMode := LISTEN_FOR_GROUP_1
          nextAction := NO_NEXT_ACTION
    LISTEN_FOR_GROUP_3..LISTEN_FOR_PASSWORD:
      Debug.Str(string(13, 13, 13, "Error, none active group selected."))
      Debug.Str(string("Switching to trigger word.", 13, 13))
      nextMode := LISTEN_FOR_WORDSET_0
      nextAction := NO_NEXT_ACTION
  ' lots of changes to this section 10/23
  {if (numericInputType == FIXED_DIGIT_INPUT and inputDigitsCountdown > 0) or {
    } numericInputType == TEN_TERMINATED_INPUT 
    nextMode := LISTEN_FOR_WORDSET_3 ' listen for numbers
  else
    TakeAction(adjustedIndex)  }
 
  if numericInputType == NO_NUMERIC_INPUT
    if listeningMode == LISTEN_FOR_WORDSET_3 ' numeric input just finished
      nextMode := afterActionListeningMode   ' saved the 
    TakeAction(adjustedIndex)
  else
    if listeningMode <> LISTEN_FOR_WORDSET_3 ' numeric input hasn't happened yet
      afterActionListeningMode := nextMode ' so listening mode can be set after
                                           ' numeric input is complete
    nextMode := LISTEN_FOR_WORDSET_3 ' listen for numbers
    
  'SetListeningMode(nextMode)
        
  priorPreviousIndex := previousIndex ' I don't think this is used
  previousIndex := adjustedIndex

PUB TakeAction(localIndex)

  case nextAction
    NO_NEXT_ACTION: return
    ACTION_NEXT: ' added 10/23 changed
      Debug.Str(string(13, "Received command to take action #"))
      Debug.Dec(numericInputTotal) ' this should be a single digit number
      Debug.Str(string(".", 13, "Add a case statement or method call here"))
      Debug.Str(string(13, "to start appropriate action. "))
      nextAction := NO_NEXT_ACTION     
    MOVE_LEFT_NEXT:  ' added 10/23 changed
      Debug.Str(string(13, "Received command to move left. "))
      Debug.Str(string(13, "Add appropriate action here. "))
      nextAction := NO_NEXT_ACTION 
    MOVE_RIGHT_NEXT:
      Servo.Set(SERVO_PIN, RIGHT_SERVO_PULSE)' added 10/23
      Debug.Str(string(13, "Received command to move right. "))
      Debug.Str(string(13, "Moving servo on pin "))
      Debug.Dec(SERVO_PIN)
      Debug.Str(string(" to position "))
      Debug.Dec(RIGHT_SERVO_PULSE)
      Debug.Char(".") 
      nextAction := NO_NEXT_ACTION 
    MOVE_UP_NEXT:  ' added 10/23 changed
      Debug.Str(string(13, "Received command to move up "))
      Debug.Dec(numericInputTotal)
      Debug.Str(string(" units"))
      Servo.Set(SERVO_PIN, CENTER_SERVO_PULSE + numericInputTotal)
      Debug.Str(string(13, "Moving servo on pin "))
      Debug.Dec(SERVO_PIN)
      Debug.Str(string(" to position "))
      Debug.Dec(CENTER_SERVO_PULSE + numericInputTotal)
      Debug.Char(".") 
      nextAction := NO_NEXT_ACTION 
    MOVE_DOWN_NEXT:
      Debug.Str(string(13, "Received command to move down. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      Debug.Str(string(13, "Add method to move appropriate amount. "))
      nextAction := NO_NEXT_ACTION 
    MOVE_FORWARD_NEXT:
      Debug.Str(string(13, "Received command to move forward. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    MOVE_BACKWARD_NEXT:
      Debug.Str(string(13, "Received command to move backward "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    TURN_LEFT_NEXT:
      Debug.Str(string(13, "Received command to turn left. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    TURN_RIGHT_NEXT:
      Debug.Str(string(13, "Received command to turn right. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    TURN_UP_NEXT:
      Debug.Str(string(13, "Received command to turn up. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    TURN_DOWN_NEXT:
      Debug.Str(string(13, "Received command to turn down. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    TURN_FORWARD_NEXT:
      Debug.Str(string(13, "Received command to turn forward. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    TURN_BACKWARD_NEXT:
      Debug.Str(string(13, "Received command to turn backward. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    RUN_NEXT:
      Debug.Str(string(13, "Received command to run program #"))
      Debug.Dec(numericInputTotal)
      Debug.Str(string(".", 13, "Add a case statement or method call here"))
      Debug.Str(string(13, "to direct to appropriate program. "))
      nextAction := NO_NEXT_ACTION
    LOOK_LEFT_NEXT:
      Debug.Str(string(13, "Received command to look left. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    LOOK_RIGHT_NEXT:
      Debug.Str(string(13, "Received command to look right. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    LOOK_UP_NEXT:
      Debug.Str(string(13, "Received command to look up. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    LOOK_DOWN_NEXT:
      Debug.Str(string(13, "Received command to look down. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    LOOK_FORWARD_NEXT:
      Debug.Str(string(13, "Received command to look forward. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    LOOK_BACKWARD_NEXT:
      Debug.Str(string(13, "Received command to look backward. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    ATTACK_LEFT_NEXT:
      Debug.Str(string(13, "Received command to attack left. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    ATTACK_RIGHT_NEXT:
      Debug.Str(string(13, "Received command to attack right. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    ATTACK_UP_NEXT:
      Debug.Str(string(13, "Received command to attack up. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    ATTACK_DOWN_NEXT:
      Debug.Str(string(13, "Received command to attack down. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    ATTACK_FORWARD_NEXT:
      Debug.Str(string(13, "Received command to attack forward. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    ATTACK_BACKWARD_NEXT:
      Debug.Str(string(13, "Received command to attack backward. "))
      Debug.Str(string(13, "Add appropriate action here. "))  
      nextAction := NO_NEXT_ACTION 
    STOP_NEXT:
      Debug.Str(string(13, "Received command to stop"))
      Debug.Str(string(13, "Take needed action to stop. "))
      nextAction := NO_NEXT_ACTION 
    HELLO_NEXT: ' I don't think this action is ever taken
      Debug.Str(string(13, "Received command to say hello."))
      Debug.Dec(numericInputTotal)
      Debug.Str(string(13, 13, "Hello", 13))
      Debug.Str(string(13, "One could also add a method to say hello. "))
      nextAction := NO_NEXT_ACTION 
    LED_NEXT:
      LedControl(localIndex)

  numericInputType := NO_NUMERIC_INPUT ' clear input type just once
  numericInputTotal := 0 ' Zero variable here so I don't need add zeroing code wherever 
                         ' numeric input is required.
  inputDigitsCountdown := 0 ' this line shouldn't be needed but it shouldn't hurt either
                            ' remove it later
                            
PUB LedControl(ledIndex)

  Debug.Str(string(13, "** "))

  case ledIndex
    RED_G2:
      Debug.Str(string("Red"))
      outa[RED_LED] := LED_ON
      outa[GREEN_LED] := LED_OFF
      outa[BLUE_LED] := LED_OFF      
    YELLOW_G2:
      Debug.Str(string("Yellow"))
      outa[RED_LED] := LED_ON
      outa[GREEN_LED] := LED_ON
      outa[BLUE_LED] := LED_OFF      
    GREEN_G2:
      Debug.Str(string("Green"))
      outa[RED_LED] := LED_OFF
      outa[GREEN_LED] := LED_ON
      outa[BLUE_LED] := LED_OFF      
    BLUE_G2:
      Debug.Str(string("Blue"))
      outa[RED_LED] := LED_OFF
      outa[GREEN_LED] := LED_OFF
      outa[BLUE_LED] := LED_ON    
    VIOLET_G2:
      Debug.Str(string("Violet"))
      outa[RED_LED] := LED_ON
      outa[GREEN_LED] := LED_OFF
      outa[BLUE_LED] := LED_ON    
    OFF_G2:
      Debug.Str(string("Off"))
      outa[RED_LED] := LED_OFF
      outa[GREEN_LED] := LED_OFF
      outa[BLUE_LED] := LED_OFF      
  Debug.Str(string(" **"))
      
PRI SetListeningMode(localMode) | localPtr, localIndex, previousMode

  previousMode := listeningMode
  listeningMode := localMode

  if listeningMode <> previousMode
    Debug.Str(string(13, "SetListeningMode("))
    Debug.Dec(listeningMode)
    Debug.Str(string(")", 13, "listeningMode = ", QUOTE))
    Debug.Str(GetListeningModeStr)
    Debug.Char(QUOTE)
  
  {localPtr := @iWordset0 ' set pointer to first word listed in DAT section
                         ' this pinter will be adjusted below
  if localMode > 0
    repeat localIndex from 0 to listeningMode - 1
      localPtr += iSetSize[localIndex] * WORDSET_LABEL_SIZE ' skip words in previous sets
 
  activeWordsPtr := localPtr  ' so other methods can find these words   }
    activeWords := iSetSize[listeningMode] ' so other methods know how many words are being
                                      ' actively listened for                                                                                                                                           
    activeWordsPtr := groupAddrTable[listeningMode]
    Debug.Str(string(13, "activeWords = "))
    Debug.Dec(activeWords)
    if listeningMode == LISTEN_FOR_NOTHING
      Debug.Str(string(13, "Not currently listening for any words."))
    else  
      Debug.Str(string(13, 13, "Say one of the following words:"))
     
    DisplayActiveWords

  case listeningMode
    LISTEN_FOR_WORDSET_0 .. LISTEN_FOR_WORDSET_3:
      TxEvrTwoCharacters(CMD_RECOG_SI, listeningMode, 1)
    LISTEN_FOR_GROUP_1 .. LISTEN_FOR_PASSWORD: 
      TxEvrTwoCharacters(CMD_RECOG_SD, listeningMode - LISTEN_FOR_WORDSET_3, 1) 
    LISTEN_FOR_NOTHING:
      TxEvrTwoCharacters(CMD_SLEEP, 0, 1)
    other:
      Debug.Str(string(13, "Not a valid listening mode value."))
      Debug.Str(string(13, "listeningMode = "))
      Debug.Dec(listeningMode)
      return
  
PUB GetNumber | localCharacter, numberEnteredFlag
''  Watch out for the abort.  If a method calling this method has
''  the SD card mounted, it needs to have an abort trap located
''  such that the any open files will be closed and the SD card
''  will be unmounted before it exits.

  numberEnteredFlag := 0
  localCharacter := 0
  'repeat while numberEnteredFlag == 0
  repeat while localCharacter <> 13
    localCharacter := Debug.CharIn
    if localCharacter == 13 and numberEnteredFlag == 0
      return -1
    elseif localCharacter == "x" or localCharacter == "X"
      Debug.Str(string("Exiting", 13)) 
      'exitFlag := 1   ' global to exit from a menu
      'abort string("Exit from GetNumber method", 13)
      return -2
    elseif localCharacter => "0" and localCharacter =< "9"  
      result := (result * 10) +  (localCharacter - 48) 
      numberEnteredFlag := 1

PUB GetListeningModeStr | localPtr, localSize

  result := localPtr := @listeningText
  localPtr += BASE_TEXT_SIZE 'strsize(@listeningText)
  case listeningMode
    LISTEN_FOR_WORDSET_0..LISTEN_FOR_WORDSET_3:
      localSize := strsize(@wordsetText)
      bytemove(localPtr, @wordsetText, localSize)
      localPtr += localSize
      byte[localPtr++] := "0" + (listeningMode - LISTEN_FOR_WORDSET_0)
    LISTEN_FOR_GROUP_1..LISTEN_FOR_GROUP_15:
      localSize := strsize(@groupText)
      bytemove(localPtr, @groupText, localSize)
      localPtr += localSize
      byte[localPtr++] := "1" + (listeningMode - LISTEN_FOR_GROUP_1)
    LISTEN_FOR_PASSWORD:
      localSize := strsize(@passwordText)
      bytemove(localPtr, @passwordText, localSize)
      localPtr += localSize
    LISTEN_FOR_NOTHING:
      localSize := strsize(@nothingText)
      bytemove(localPtr, @nothingText, localSize)
      localPtr += localSize
  byte[localPtr] := 0 ' terminate string
     
PUB DisplayActiveWords | localPtr, localIndex

  localPtr := activeWordsPtr 
  localIndex := 0                                                                                                                                 
  repeat activeWords ' display the words in this set
    Debug.Char(13)
    Debug.Dec(localIndex++)
    Debug.Char(")")
    Debug.Char(32)
    Str(localPtr) 
    localPtr += WORDSET_LABEL_SIZE
  Debug.Char(13)
  
PUB DisplayError(localCharacter, lowest, highest)

  Debug.Str(string(13, "Error"))
  if lowest == highest
    Debug.Str(string(13, "Expected value was ", QUOTE))
    SafeTx(lowest)
    Debug.Char(QUOTE)
  else
    Debug.Str(string(13, "Expected values were between ", QUOTE))
    SafeTx(lowest)
    Debug.Str(string(QUOTE, " and ", QUOTE))
    SafeTx(highest)
    Debug.Char(QUOTE)
  Debug.Str(string(13, "The value received was ", QUOTE))
  SafeTx(localCharacter)
  Debug.Char(QUOTE)
     
PUB ReceiveReply | localCharacter

  bytefill(@tempBufferRx, 0, TEMP_BUFFER_SIZE)
  repeat
    localCharacter := RxTime(EASYVR_COM, RX_TIME_WAIT)
    if localCharacter <> -1
      EasyVr.Char(32)
      tempBufferRx[result++] := localCharacter
  while localCharacter <> -1
  
PUB Str(localPtr)

  result := strsize(localPtr)
  repeat result
    SafeTx(byte[localPtr++])

PUB SafeTx(localCharacter)

  if localCharacter => 33 and localCharacter =< "~"
    Debug.Char(localCharacter)
  else
    Debug.Char("<")
    Debug.Char("$")
    Debug.Hex(localCharacter, 2)
    Debug.Char(">")
    
PRI RxTime(comLine, timeToWait) | localTime

  localTime := clkfreq / 1000
  timeToWait *= localTime
  localTime := cnt
  repeat
    if comLine == DEBUG_COM
      result := Debug.RxCount
    else
      result := EasyVr.RxCount  
  while cnt - localTime < timeToWait and result == 0
  
  if result
    if comLine == DEBUG_COM
      result := Debug.CharIn
    else
      result := EasyVr.CharIn
  else
    result := -1    
  
PRI FillSetSizeTable
'' Compute the number of words in each wordset or group.

  FixAddressTable
  repeat result from 0 to LAST_LISTENING_MODE_INDEX - 1
    iSetSize[result] := groupAddrTable[result + 1] - groupAddrTable[result]
    iSetSize[result] /= WORDSET_LABEL_SIZE

  iSetSize[LAST_LISTENING_MODE_INDEX] := 1
    
PRI FixAddressTable | addressAdjust
'' Adjust the stored addresses for the offset amount (usually 16).

  addressAdjust := @iWordset0 - groupAddrTable[0]
  if addressAdjust == 0
    return 0
  repeat result from 0 to LAST_LISTENING_MODE_INDEX 
    groupAddrTable[result] += addressAdjust
    
DAT

'' Sizes are now computed at runtime.

iSetSize                byte 0[LISTENING_MODES]


groupAddrTable          word @iWordset0, @iWordset1, @iWordset2, @iWordset3, @group1, @group2
                        word @group3, @group4, @group5, @group6, @group7, @group8, @group9
                        word @group10, @group11, @group12, @group13, @group14, @group15
                        word @password, @noWord
                                    
'' Each word in a wordset has eleven characters including trailling zero(s).
'' This limits the length of a word to ten letters. This is the same limit
'' allowed by the EasyVR GUI.
' It looks like "backward" currently has the most characters with eight letters.
' With the constant "WORDSET_LABEL_SIZE" set to it's current value of eleven, a word
' may have up to ten letters. If one wished to use larger words, the value of
' the constant "WORDSET_LABEL_SIZE" would need to be changed. The value of
' "WORDSET_LABEL_SIZE" needs to set to at least one greater than the length
' of the longest word used.


                            '012345678
iWordset0               byte "robot", 0[WORDSET_LABEL_SIZE - 5]
         
                            '0123456789
iWordset1               byte "action", 0[WORDSET_LABEL_SIZE - 6]
                        byte "move", 0[WORDSET_LABEL_SIZE - 4]
                        byte "turn", 0[WORDSET_LABEL_SIZE - 4]
                        byte "run", 0[WORDSET_LABEL_SIZE - 3]
                        byte "look", 0[WORDSET_LABEL_SIZE - 4]
                        byte "attack", 0[WORDSET_LABEL_SIZE - 6]
                        byte "stop", 0[WORDSET_LABEL_SIZE - 4]
                        byte "hello", 0[WORDSET_LABEL_SIZE - 5]

iWordset2               byte "left", 0[WORDSET_LABEL_SIZE - 4]
                        byte "right", 0[WORDSET_LABEL_SIZE - 5]
                        byte "up", 0[WORDSET_LABEL_SIZE - 2]
                        byte "down", 0[WORDSET_LABEL_SIZE - 4]
                        byte "forward", 0[WORDSET_LABEL_SIZE - 7]
                        byte "backward", 0[WORDSET_LABEL_SIZE - 8]
        
iWordset3               byte "zero", 0[WORDSET_LABEL_SIZE - 4]
                        byte "one", 0[WORDSET_LABEL_SIZE - 3]
                        byte "two", 0[WORDSET_LABEL_SIZE - 3]
                        byte "three", 0[WORDSET_LABEL_SIZE - 5]
                        byte "four", 0[WORDSET_LABEL_SIZE - 4]
                        byte "five", 0[WORDSET_LABEL_SIZE - 4]
                        byte "six", 0[WORDSET_LABEL_SIZE - 3]
                        byte "seven", 0[WORDSET_LABEL_SIZE - 5]
                        byte "eight", 0[WORDSET_LABEL_SIZE - 5]
                        byte "nine", 0[WORDSET_LABEL_SIZE - 4]
                        byte "ten", 0[WORDSET_LABEL_SIZE - 3]

group1                  byte "LED", 0[WORDSET_LABEL_SIZE - 3]
                        byte "other", 0[WORDSET_LABEL_SIZE - 5]
              
group2                  byte "red", 0[WORDSET_LABEL_SIZE - 3]
                        byte "yellow", 0[WORDSET_LABEL_SIZE - 6]
                        byte "green", 0[WORDSET_LABEL_SIZE - 5]
                        byte "blue", 0[WORDSET_LABEL_SIZE - 4]
                        byte "violet", 0[WORDSET_LABEL_SIZE - 6]
                        byte "off", 0[WORDSET_LABEL_SIZE - 3]
                        byte "exit", 0[WORDSET_LABEL_SIZE - 4]
'' If additional groups are used, you may want to add constants to
'' enumerate the word choices.    
group3
group4
group5
group6
group7
group8
group9
group10
group11
group12
group13
group14
group15
password

noWord                  byte "no word", 0[WORDSET_LABEL_SIZE - 7]

                           
              '012345678
wordError               byte "Error.  Word not found.", 0

DAT ' listeningMode text
                             '123456789012345678901
listeningText           byte "LISTEN_FOR_", 0[LISTENING_TEXT_SIZE - BASE_TEXT_SIZE] 
wordsetText             byte "WORDSET_", 0
groupText               byte "GROUP_", 0
passwordText            byte "PASSWORD", 0
nothingText             byte "NOTHING", 0
                               